import logging
import random
from typing import Dict, List, Tuple, cast

from faker import Faker
from locust import events, task

from performance_tests.common import (
    ProvisioningClient,
    ResponseError,
    get_id_broker_schools,
    get_provisioning_api_admins,
)

user_credentials: List[Tuple[str, str]] = []  # [(username, password), ..]
id_broker_schools: Dict[str, Dict[str, str]] = {}  # {sch_auth: {ou: uuid, ...}}
fake = Faker()


@events.init.add_listener
def on_init(environment, **kwargs):
    creds = get_provisioning_api_admins()
    credentials = cast(List[Tuple[str, str]], sorted(creds.items()))
    user_credentials.extend(credentials)
    id_broker_schools.update(get_id_broker_schools())


class ProvisioningAPIClient(ProvisioningClient):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.auth_username, self.auth_password = random.choice(user_credentials)
        self.school_authority = self.auth_username.rsplit("-", 1)[-1]
        self.schools = list(id_broker_schools[self.school_authority].items())

    @task
    def school_class_create(self) -> None:
        school, _ = random.choice(self.schools)
        name = f"test.{fake.user_name()[:10]}{fake.pyint(1000, 9999)}"
        entry_uuid = fake.uuid4()
        try:
            super().school_class_create(
                school_authority=self.school_authority,
                obj_id=entry_uuid,
                name=name,
                school=school,
                description="Performance Test temp class",
                members=[],
            )
        except ResponseError as exc:
            logging.error(
                "%s: school_authority=%r name=%r entry_uuid=%r -> %r",
                self.greenlet.name,
                self.school_authority,
                name,
                entry_uuid,
                exc,
            )
