import logging
import random
from typing import List, Tuple, cast

from faker import Faker
from locust import events, task

from performance_tests.common import ProvisioningClient, ResponseError, get_provisioning_api_admins

user_credentials: List[Tuple[str, str]] = []  # [(username, password), ..]
fake = Faker()


@events.init.add_listener
def on_init(environment, **kwargs):
    creds = get_provisioning_api_admins()
    credentials = cast(List[Tuple[str, str]], sorted(creds.items()))
    user_credentials.extend(credentials)


class ProvisioningAPIClient(ProvisioningClient):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.auth_username, self.auth_password = random.choice(user_credentials)
        self.school_authority = self.auth_username.rsplit("-", 1)[-1]

    @task
    def school_create(self) -> None:
        name = f"test.{fake.user_name()[:10]}{fake.pyint(1000, 9999)}"
        entry_uuid = fake.uuid4()
        try:
            super().school_create(
                school_authority=self.school_authority,
                obj_id=entry_uuid,
                name=name,
                display_name="Performance Test temp OU",
            )
        except ResponseError as exc:
            logging.error(
                "%s: school_authority=%r name=%r entry_uuid=%r -> %r",
                self.greenlet.name,
                self.school_authority,
                name,
                entry_uuid,
                exc,
            )
